<?php

/**
 * Make variations that are unavailable be grayed out and unselectable. Won't apply to items that allow backorder.
 */
add_filter( 'woocommerce_variation_is_active', function ( $active, $variation ) {
	if ( ! $variation->is_in_stock() ) {
		return false;
	}
	return $active;
}, 9988, 2 );


/**
 * Filter product variation attribute option (in add to cart form) args to:
 * - disable the "None" option, if set via bt-ecomm setup/options
 */
add_filter( 'woocommerce_dropdown_variation_attribute_options_args', function( $args ) {
	//error_log( __METHOD__ . ' -- woocommerce_dropdown_variation_attribute_options_args = ' . print_r( $args, true ) );
	if ( bt_ecomm_option( 'omit_variation_option_none' ) ) {
		$args['show_option_none'] = false;
	}
	return $args;
} );


/**
 * Filter product variation options reset link (in add to cart form) to:
 * - omit the link altogether, if set via bt-ecomm setup/options
 */
add_filter( 'woocommerce_reset_variations_link', function( $link_html ) {
	//error_log( __METHOD__ . ' -- woocommerce_reset_variations_link = ' . print_r( $link_html, true ) );
	if ( bt_ecomm_option( 'omit_variation_option_none' ) ) {
		$link_html = '';
	}
	return $link_html;
} );


/**
 * Filter product price html to:
 * - omit it, if there are any product ids set for this in bt-ecomm setup/options
 */
add_filter( 'woocommerce_get_price_html', function( $price, $product ) {
	//error_log( __METHOD__ . ' -- price html = ' . $price . ' // for product ' . print_r( $product, true ) );

	// omit price for any products defined as such
	if ( $omit_price_for_products = bt_ecomm_option( 'omit_price_for_products' ) ) {
		//error_log( __METHOD__ . ' -- omit_price_for_products = ' . print_r( $omit_price_for_products, true ) . ' / on: ' . get_the_title() );
		if ( in_array( get_the_ID(), $omit_price_for_products ) ) {
			return '';
		}
	}

	return $price;
}, 2, 9988 );


/**
 * Filter product tabs to:
 * - omit any specific tabs set in bt-ecomm setup/options
 */
add_filter( 'woocommerce_product_tabs', function( $tabs ) {
	//error_log( __METHOD__ . ' -- woocommerce_product_tabs = ' . print_r( $tabs, true ) );

	if ( $omit_specific_tabs = bt_ecomm_option( 'omit_woocommerce_specific_product_tabs' ) ) {
		foreach ( $omit_specific_tabs as $omit_tab ) {
			if ( array_key_exists( $omit_tab, $tabs ) ) {
				//error_log( __METHOD__ . '   -- omitting tab = ' . $omit_tab );
				unset( $tabs[$omit_tab] );
			}
		}
	}

	return $tabs;
}, 9988 );


/**
 * On admin init:
 * - unregister product category taxonomy, if disabled via theme-supports
 * - unregister product tag taxonomy, if disabled via theme-supports
 */
add_action( 'admin_init', function() {
	//error_log( 'ADMIN INIT: user = ' . wp_get_current_user()->user_email . ' / bt_is_seller() = ' . ( bt_is_seller() ? 'true' : 'false' ) );

	if ( bt_use_sellers( 'disable_product_categories' ) ) {
		unregister_taxonomy( 'product_cat' );
	}

	if ( bt_use_sellers( 'disable_product_tags' ) ) {
		unregister_taxonomy( 'product_tag' );
	}

}, 9988, 0 );


/**
 * On admin head, for single-page shops:
 * - remove meta box for product post excerpt
 * - remove meta box for product images/gallery
 */
add_action( 'admin_head', function() {
	//error_log( 'ADMIN HEAD: user = ' . wp_get_current_user()->user_email . ' / bt_is_seller() = ' . ( bt_is_seller() ? 'true' : 'false' ) );

	// check if we use single-page shop
	if ( bt_use_sellers( 'seller_single_page_shop' ) ) {
		remove_meta_box( 'postexcerpt', 'product', 'normal' );
		remove_meta_box( 'woocommerce-product-images', 'product', 'side' );
	}

}, 9988, 0 );


/**
 * Filter product type selector to remove Grouped and External options, if we use sellers.
 */
add_filter( 'product_type_selector', function( $types ) {
	//error_log( 'PRODUCT TYPES: types BEFORE = ' . print_r( $types, 1 ) );

	if ( bt_use_sellers() ) {
		// remove "Grouped product" option
		if ( isset( $types[ 'grouped' ] ) ) {
			unset( $types[ 'grouped' ] );
		}

		// remove "External/Affiliate product" option unless it's specifically allowed
		if ( isset( $types[ 'external' ] ) && ! bt_use_sellers( 'allow_external_products') ) {
			unset( $types[ 'external' ] );
		}

		// remove subscription options for sellers (allowed for admins e.g. nemaa memberships
		if ( bt_is_seller() ) {
			if ( isset( $types[ 'subscription' ] ) ) {
				unset( $types[ 'subscription' ] );
			}
			if ( isset( $types[ 'variable-subscription' ] ) ) {
				unset( $types[ 'variable-subscription' ] );
			}
		}
	}

	//error_log( 'PRODUCT TYPES: types AFTER = ' . print_r( $types, 1 ) );
	return $types;

}, 9988 );


/**
 * Filter product type options to remove Virtual and Downloadable options, if we use sellers.
 *
 * @see https://rudrastyh.com/woocommerce/remove-virtual-and-downloadable.html for how to remove from index filters too
 */
add_filter( 'product_type_options', function( $options ) {
	//error_log( 'PRODUCT TYPE OPTIONS: options BEFORE = ' . print_r( $options, 1 ) );

	if ( bt_use_sellers() ) {
		// remove "Virtual" checkbox
		if ( isset( $options['virtual'] ) && bt_is_seller() ) {
			unset( $options['virtual'] );
		}

		// remove "Downloadable" checkbox
		if ( isset( $options['downloadable'] ) ) {
			unset( $options['downloadable'] );
		}
	}

	//error_log( 'PRODUCT TYPE OPTIONS: options AFTER = ' . print_r( $options, 1 ) );
	return $options;

}, 9988 );


/**
 * Filter product data tabs in admin to :
 * - remove Linked Products, Advanced, Marketplace Suggestions.
 */
add_filter( 'woocommerce_product_data_tabs', function( $tabs ) {
	//error_log( 'PRODUCT DATA TABS: tabs BEFORE = ' . print_r( $tabs, 1 ) );

	// remove "Linked Products" tab
	if ( isset( $tabs[ 'linked_product' ] ) ) {
		unset( $tabs[ 'linked_product' ] );
	}

	// remove "Advanced" tab
	if ( isset( $tabs[ 'advanced' ] ) ) {
		unset( $tabs[ 'advanced' ] );
	}

	// remove "Subscriptions" (Subscribe all the Things) and "Custom Fields" tabs
	if ( bt_is_seller() ) {
		if ( isset( $tabs[ 'satt' ] ) ) {
			unset( $tabs[ 'satt' ] );
		}
		if ( isset( $tabs[ 'customfields' ] ) ) {
			unset( $tabs[ 'customfields' ] );
		}
	}

	// remove "Get more options" tab
	if ( isset( $tabs[ 'marketplace-suggestions' ] ) ) {
		unset( $tabs[ 'marketplace-suggestions' ] );
	}

	//error_log( 'PRODUCT DATA TABS: tabs AFTER = ' . print_r( $tabs, 1 ) );
	return $tabs;

}, 9988 );


/**
 * Filter default WC Product export columns to:
 * - add author column
 */
add_filter( 'woocommerce_product_export_product_default_columns', function ( $columns ) {
	//error_log( __METHOD__ . ' -- columns = ' . print_r( $columns, true ) );

	if ( ! isset( $columns['author'] ) ) {
		$columns['author'] = 'Author';
	}

	return $columns;
} );


/**
 * Set post author data in WC Product export
 */
add_filter( 'woocommerce_product_export_product_column_author', function( $blank, $product, $column_id ) {
	//error_log( __METHOD__ . ' -- blank = ' . $blank . ' -- column_id = ' . $column_id . ' -- product = ' . print_r( $product, true ) );
	//error_log( __METHOD__ . ' -- post id = ' . $product->get_id() . ' -- author: ' . get_post_field( 'post_author', $product->get_id() ) );

	return get_post_field( 'post_author', $product->get_id() );
}, 9988, 3 );


/**
 * Filter product loop start html to:
 * - use custom html if setup/options have it.
 */
add_filter( 'woocommerce_product_loop_start', function( $html ) {
	if ( $custom_html = bt_ecomm_option( 'woocommerce_product_loop_start' ) ) {
		$html = $custom_html;
	}
	return $html;
} );


/**
 * Filter product loop end html to:
 * - use custom html if setup/options have it.
 */
add_filter( 'woocommerce_product_loop_end', function( $html ) {
	if ( $custom_html = bt_ecomm_option( 'woocommerce_product_loop_end' ) ) {
		$html = $custom_html;
	}
	return $html;
} );


/**
 * Alter stock html:
 * - use custom out of stock html if it exists
 * - if not out of stock, omit the message if bt-ecomm option is set
 */
add_filter( 'woocommerce_get_stock_html', function ( $html, $product ) {
	//error_log( __METHOD__ . ' -- ORIGINAL html = ' . print_r( $html, true ) );
	//error_log( __METHOD__ . ' -- product = ' . $product->get_title() );

	if ( strpos( $html, 'Out of stock' ) !== false ) {
		// OUT OF STOCK
		if ( $custom_html = bt_ecomm_option( 'out_of_stock_custom_html' ) ) {
			$html = $custom_html;
		}
	} else if ( bt_ecomm_option( 'omit_in_stock_message' ) ) {
		// IN STOCK AND OPTION IS SET TO OMIT THE MESSAGE
		$html = '';
	}

	//error_log( __METHOD__ . ' -- FINAL html = ' . print_r( $html, true ) );
	return $html;

}, 9988, 2 );
